@@ -3,7 +3,7 @@
 # foomatic.py - Pythonic wrappers for foomatic-configure
 #
 #     foomatic-gui - GNOME2 interface to the foomatic printing system
-#     Copyright (C) 2002-03 Chris Lawrence <lawrencc@debian.org>
+#     Copyright (C) 2002-04 Chris Lawrence <lawrencc@debian.org>
 #
 #     This program is free software; you can redistribute it and/or modify
 #     it under the terms of the GNU General Public License as published by
@@ -19,7 +19,7 @@
 #     along with this program; if not, write to the Free Software
 #     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
-# $Id: foomatic.py,v 1.28 2004/06/08 05:30:27 lordsutch Exp $
+# $Id: foomatic.py,v 1.47 2010/05/23 00:32:34 lordsutch Exp $
 
 import commands
 from commands import mkarg
@@ -28,7 +28,12 @@ import os
 import re
 import sys
 import xml.sax
+import pprint    
+import urlutils
+import urlparse
+import gzip
 
+# Only run in debug mode on my machines... cheesy, yes.
 DEBUG=os.path.exists('/home/cnlawren') or os.path.exists('/home/quango')
 
 class FoomaticXMLHandler(xml.sax.ContentHandler):
@@ -81,12 +86,13 @@ class FoomaticQueueHandler(FoomaticXMLHandler):
         else:
             FoomaticXMLHandler.endElement(self, name)
 
-AUTODETECT_TYPES = ('general', 'parallel', 'serial')
+AUTODETECT_TYPES = ('general', 'parallel', 'usb', 'serial', 'snmp')
 class FoomaticOverviewHandler(FoomaticXMLHandler):
     def __init__(self):
         FoomaticXMLHandler.__init__(self)
         self.printers = {}
         self.makes = {}
+        self.cmakes = {} # All caps make and model
         self.autodetect_ids = {}
         self._drivers_block = self._inautoblock = False
 
@@ -113,15 +119,20 @@ class FoomaticOverviewHandler(FoomaticXMLHandler):
 
             if make and model:
                 self.makes.setdefault(make, {})[model] = self._curinfo
+                # Also include uppercase make and model for IEEE 1284 ad-hoc
+                # detection support
+                self.cmakes.setdefault(make.upper(), {})[model.upper()] = self._curinfo
 
             if id:
                 self.printers[id] = self._curinfo
 
             if autodetect:
-                for info in AUTODETECT_TYPES:
-                    x = autodetect.get(info)
+                for x in autodetect.values():
+                    if x and ('model' in x and 'manufacturer' in x):
+                        self.autodetect_ids[(x['manufacturer'], x['model'])] = self._curinfo
                     if x and 'description' in x:
                         self.autodetect_ids[x['description']] = self._curinfo
+            #pprint.pprint(self._curinfo)
             self._curinfo = {}
         elif name == 'drivers':
             self._curinfo[name] = self.drivers
@@ -138,6 +149,8 @@ class FoomaticOverviewHandler(FoomaticXMLHandler):
             self._inautoblock = False
             del self.autodetect
         elif name in AUTODETECT_TYPES:
+            if not hasattr(self, 'autodetect'):
+                self.autodetect = {}
             self.autodetect[name] = self._block
             self._block = self._curinfo
         else:
@@ -147,7 +160,7 @@ class FoomaticPrinterHandler(FoomaticXMLHandler):
     def __init__(self):
         FoomaticXMLHandler.__init__(self)
         self.make = self.model = self.url = self.functionality = self.driver =\
-                    self.contrib_url = ''
+                    self.contrib_url = self.ppd = ''
         self.mechanism = {}
         self.languages = {}
         self.autodetect = {}
@@ -178,7 +191,7 @@ class FoomaticPrinterHandler(FoomaticXMLHandler):
         if name == 'printer':
             pass
         elif name in ('make', 'model', 'url', 'functionality', 'driver',
-                      'contrib_url'):
+                      'contrib_url', 'ppd'):
             setattr(self, name, self._data.strip())
         elif name == 'verified':
             self.verified = True
@@ -230,37 +243,51 @@ def parse_foomatic(commandline, handler):
 
 def get_printer_queues():
     handler = FoomaticQueueHandler()
-    parse_foomatic('foomatic-configure -q -Q -r 2>/dev/null', handler)
+    try:
+        parse_foomatic('foomatic-configure -q -Q -r 2>/dev/null', handler)
+    except:
+        return None
     return handler
 
 def get_printer_db():
     handler = FoomaticOverviewHandler()
-    parse_foomatic('foomatic-configure -q -O 2>/dev/null', handler)
+    try:
+        parse_foomatic('foomatic-configure -q -O 2>/dev/null', handler)
+    except:
+        return None
     return handler
 
 # Parse the xml printer and driver information - not done yet
 # A wrapper for "foomatic-configure -X -p %s"
 def get_printer_info(printer):
     handler = FoomaticPrinterHandler()
-    parse_foomatic('foomatic-configure -X -p' + mkarg(printer) + " 2>/dev/null", handler)
+    try:
+        parse_foomatic('foomatic-configure -X -p' + mkarg(printer) + " 2>/dev/null", handler)
+    except:
+        return None
     return handler
 
 def get_driver_info(driver):
     handler = FoomaticDriverHandler()
-    parse_foomatic('foomatic-configure -X -d' + mkarg(driver) + ' 2>/dev/null', handler)
+    try:
+        parse_foomatic('foomatic-configure -X -d' + mkarg(driver) + ' 2>/dev/null', handler)
+    except:
+        return None
     return handler
 
 def set_default_queue(queuename):
+    mask = os.umask(0755)
     os.system('foomatic-configure -q -D -n' + mkarg(queuename) + ' 2>/dev/null')
+    os.umask(mask)
 
 def remove_queue(queuename):
     os.system('foomatic-configure -q -R -n' + mkarg(queuename) + ' 2>/dev/null')
 
 # Should be a dictionary, like those returned in get_printer_queues().queues
 def setup_queue(queueinfo):
-    if 'location' not in queueinfo:
+    if not queueinfo.get('location'):
         queueinfo['location'] = ''
-    if 'description' not in queueinfo:
+    if not queueinfo.get('description'):
         queueinfo['description'] = ''
 
     args = [('-n', 'name'),
@@ -272,12 +299,17 @@ def setup_queue(queueinfo):
         args += [('--ppd', 'ppdfile')]
     else:
         args += [('-p', 'printer'), ('-d', 'driver')]
-    
-    argstr = ' '.join([ a + mkarg(queueinfo[b]) for (a,b) in args])
-    command = 'foomatic-configure -f -q '+argstr+' 2>/dev/null'
+
+    if queueinfo.get('spooler'):
+	args += [('-s', 'spooler')]   
+ 
+    argstr = ' '.join([ a + mkarg(queueinfo[b] or '') for (a,b) in args])
+    command = 'foomatic-configure -f -q -w '+argstr #+' 2>/dev/null'
     if DEBUG:
         print command
+    mask = os.umask(0755)
     os.system(command)
+    os.umask(mask)
 
 def send_test_page(template, queueinfo):
     SUBSTITUTIONS = {'QUEUENAME' : 'name', 'PRINTER' : 'printer',
@@ -298,14 +330,18 @@ def send_test_page(template, queueinfo):
     command = 'foomatic-printjob -P' + mkarg(queueinfo['name'])
     if DEBUG:
         print command
-        
-    pipe = os.popen(command, 'w')
-    pipe.write(data)
-    pipe.close()
+
+    try:
+        pipe = os.popen(command, 'w')
+        pipe.write(data)
+        pipe.close()
+    except IOError, x:
+        return False
 
 _mappingre = re.compile(r'(\'[.0-9A-Za-z_-]*?\') => ', re.DOTALL)
 _refre = re.compile(r'{(\'[.0-9A-Za-z_-]*?\')}', re.DOTALL)
 _quotere = re.compile(r"'(([^\\]|\\.)*?)'", re.DOTALL)
+_xre = re.compile(r"\\x{([a-f0-9]+)}")
 
 def convert_perl_data(content):
     "Convert the output of foomatic-configure -P into Python.  May be ugly."
@@ -314,6 +350,7 @@ def convert_perl_data(content):
     content = content.replace('$QUEUES[0]', 'QUEUES[0]')
     content = _mappingre.sub(r'\1 : ', content)
     content = _refre.sub(r'[\1]', content)
+    content = _xre.sub(r'\\x\1', content)
     # Triple-quote all strings, since Adobe PPDs can contain multiline literals
     content = _quotere.sub(r"'''\1'''", content)
     #content = content.replace("'", "'''")
@@ -400,21 +437,79 @@ def set_printer_options(queueinfo=None, options=None, queuename='',
     for (name, val) in options.iteritems():
         settings.append( '-o' + mkarg("%s=%s" % (name, val)) )
 
-    #if ppdfile:
-    #    settings.extend( ['--ppd' + mkarg(ppdfile),
-    #                      '-d PostScript'] )
-
     command = 'foomatic-configure -q -n' + mkarg(queuename) + ' ' + (' '.join(settings)) # + ' 2>/dev/null'
     if DEBUG:
         print command
     os.system(command)
 
-def test():
-    import pprint
+def guess_spooler(pqdb=None):
+    if not pqdb:
+        pqdb = get_printer_queues()
+
+    if pqdb.spooler: return pqdb.spooler
+
+    if os.path.exists('/etc/foomatic/defaultspooler'):
+        try:
+            return open('/etc/foomatic/defaultspooler', 'r').readline().strip()
+        except:
+            pass
+
+    # Implement more detection here eventually...
+    if os.path.exists('/etc/cups'):
+        return 'cups'
+
+    return None
+
+def get_ppd_content(uri, network=True):
+    if not uri.startswith('http:'):
+        # Newer XML omits the full URI
+        if not uri.startswith('PPD/'):
+            # No idea what to do...
+            print >> sys.stderr, 'Nonstandard PPD location', uri
+            return None
+
+        filename = uri[4:]
+        filename = os.path.join('/usr/share/ppd/postscript/', uri[4:])
+        if os.path.exists(filename):
+            try:
+                fp = open(filename)
+                content = fp.read()
+                fp.close()
+                return content
+            except IOError:
+                pass
+        if os.path.exists(filename+'.gz'):
+            try:
+                fp = gzip.open(filename+'.gz')
+                content = fp.read()
+                fp.close()
+                return content
+            except IOError:
+                pass
+        # Try the website now
+        uri = urlparse.urljoin('http://openprinting.org/foomatic-db/db/source/', uri)
     
+    if not network:
+        print >> sys.stderr, 'PPD not available offline:', uri
+        return None
+
+    print >> sys.stderr, 'Fetching PPD file from', uri
+    fp = urlutils.open_url(uri)
+    if fp:
+        try:
+            content = fp.read()
+        except:
+            fp.close()
+            return None
+        fp.close()
+        return content
+    return None
+
+def test():
     # Assemble queue list
     pqdb = get_printer_queues()
     queues = pqdb.queues
+    print 'Spooler:', repr(pqdb.spooler)
     print 'Available printer queues:'
     for queue in queues:
         if 'description' in queue:
@@ -440,6 +535,8 @@ def test3():
     pprint.pprint(data.__dict__)
     print 'Canon BJC-2110 information:'
     pprint.pprint(makes['Canon']['BJC-2110'])
+    print 'HP LaserJet 2200 information:'
+    pprint.pprint(makes['HP']['LaserJet 2200'])
     print 'Autodetect information:'
     pprint.pprint(autodetect['Hewlett-Packard PhotoSmart P1100'])
     #print get_printer_info('HP-LaserJet_2200').__dict__
@@ -451,4 +548,5 @@ def test2():
     pprint.pprint(get_printer_options(None, 'lw8500', True))
 
 if __name__ == '__main__':
-    test3()
+    #print guess_spooler()
+    test()
